from datetime import date, datetime


def date_plus_hours(date_var: date, hours: str) -> datetime:
    """Returns a datetime object from a date string and a time string.

    Args:
        date_var (date): Date in date format.
        hours (str): Time in HH:MM format.

    Returns:
        datetime: Datetime object.

    Examples:
        >>> date_plus_hours(date(2021, 1, 1), '12:00')
        datetime.datetime(2021, 1, 1, 12, 0)
    """
    hours = hours[:5] + ':00'
    return datetime.fromisoformat(f'{date_var}T{hours}')



from datetime import date

from playwright.async_api import async_playwright

from ...utils import date_plus_hours


async def decolar_flights(origin, destination, flight_date, passengers=1):
    flight_date = date.fromisoformat(flight_date)

    cluster_css = '.COMMON'
    rows_locator = 'li[class="itinerary-container"]'
    carr_locator = 'span[class="airline-container airline-logo-name-container -have-name"]'

    flights = []
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True, slow_mo=0.5)
        page = await browser.new_page()
        await page.goto(
            'https://www.decolar.com/passagens-aereas/',
            wait_until='load',
            timeout=120000,
        )
        await page.get_by_role('emphasis').filter(
            has_text='Não quero benefícios'
        ).click()
        await page.click('text="Só ida"')
        date_text_locator = 'div[class="sbox5-monthgrid"]'
        origin_input = page.get_by_role(
            'textbox', name='Insira sua cidade de origem'
        )
        dest_input = page.get_by_role(
            'textbox', name='Insira sua cidade de destino'
        )
        date_input = page.get_by_role('textbox', name='Ida', exact=True)
        submit = page.get_by_role('button', name='Buscar')
        await origin_input.click()
        await origin_input.clear()
        await origin_input.type(origin)
        await page.wait_for_timeout(10000)
        await origin_input.press('Enter')
        await page.wait_for_timeout(10000)
        await dest_input.click()
        await dest_input.type(destination, delay=100)
        await page.wait_for_timeout(10000)
        await dest_input.press('Enter')
        await date_input.click()
        standard_date = await page.locator(
            date_text_locator
        ).first.get_attribute('data-month')
        standard_date = date(
            int(standard_date[0:4]), int(standard_date[5:7]), 1
        )
        while (
            standard_date.month != flight_date.month
            or standard_date.year != flight_date.year
        ):
            await page.locator('a[class="calendar-arrow-right"]').first.click()
            standard_date = await page.locator(
                date_text_locator
            ).first.get_attribute('data-month')
            standard_date = date(
                int(standard_date[0:4]), int(standard_date[5:7]), 1
            )

        await page.get_by_text(str(flight_date.day), exact=True).first.click()
        await page.wait_for_timeout(10000)
        await submit.click()
        await page.wait_for_load_state()
        await page.get_by_role('emphasis').filter(
            has_text='Não quero benefícios'
        ).click()
        await page.wait_for_selector(cluster_css, timeout=120000)
        clusters = await page.locator(cluster_css).all()
        for cluster in clusters:
            rows = await cluster.locator(rows_locator).all()
            price = cluster.locator('p[class="item-fare fare-price"]')
            price = await price.locator(
                'span[class="amount price-amount"]'
            ).text_content()
            for row in rows:
                departs = date_plus_hours(
                    flight_date, await row.locator('.leave').text_content()
                )
                arrives = date_plus_hours(
                    flight_date, await row.locator('.arrive').text_content()
                )

                carrier = await (row.locator(carr_locator).text_content())

                flight = {}
                flight['origin'] = origin
                flight['destination'] = destination
                flight['date'] = flight_date.isoformat()
                flight['carrier'] = carrier.strip()
                flight['departure_date'] = departs
                flight['arrive_date'] = arrives
                flight['price'] = price
                flights.append(flight)

    return flights



import asyncio
from datetime import date
from typing import List

from dateutil.relativedelta import relativedelta

from .models import Flight
from .scrapers.flights import scrape_flights

now_isoformat = (date.today() + relativedelta(days=1)).isoformat()


def flights(
    origin: str, destination: str, date: str, passengers: int = 1
) -> List[Flight]:
    """Returns a list of flight prices from origin to destination on date.

    Args:
        origin (str): Origin airport code.
        destination (str): Destination airport code.
        date (str): Date in YYYY-MM-DD format (iso format).
        passengers (int, optional): Number of passengers. Defaults to 1.

    Returns:
        list: List of flight prices.

    Raises:
        ValueError: If date is not in YYYY-MM-DD format (iso format).

    Examples:
        >>> flights('CWB', 'POA', '{}'.format(now_isoformat))
        [...]
    """   # doctest: +ELLIPSIS

    origin = origin.upper()
    destination = destination.upper()

    if not (date[4] == '-' and date[7] == '-' and len(date) == 10):
        raise ValueError('Date must be in YYYY-MM-DD format (iso format).')

    if not asyncio.get_event_loop().is_running():
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    flights = loop.run_until_complete(
        scrape_flights(origin, destination, date)
    )
    return flights



import locale
from typing import List

import typer
from rich.console import Console
from rich.table import Table
from typer import Option

from .flights import flights as get_flights
from .models import Flight

locale.setlocale(locale.LC_ALL, 'pt_BR.UTF-8')
app = typer.Typer()


@app.command()
def flights(
    origin: str = Option(
        ..., help='Origin airport code', prompt='Origin airport code'
    ),
    destination: str = Option(
        ..., help='Destination airport code', prompt='Destination airport code'
    ),
    date: str = Option(
        ...,
        prompt='Date in YYYY-MM-DD format (iso format)',
        help='Date in YYYY-MM-DD format (iso format) e.g. 2021-01-01',
    ),
    passengers: int = Option(1, help='Number of passengers'),
):
    table = Table(title='Flights')
    console = Console()

    table.add_column('Origem')
    table.add_column('Destino')
    table.add_column('Embarque')
    table.add_column('Chegada')
    table.add_column('Preço')
    table.add_column('Companhia')
    flights_list: List[Flight] = get_flights(
        origin, destination, date, passengers
    )

    for flight in flights_list:
        table.add_row(
            flight.origin,
            flight.destination,
            flight.departure_date.strftime('%d/%m/%Y às %H:%M'),
            flight.arrive_date.strftime('%d/%m/%Y às %H:%M'),
            locale.currency(flight.price, grouping=True),
            flight.carrier,
        )

    console.print(table)



import uuid
from datetime import datetime as dt

from pydantic import BaseModel, validator


class Flight(BaseModel):
    """Flight model.
    >>> Flight(id=uuid.UUID('50b5e710-db51-424c-b052-8383635ad39d'), origin='CWB', destination='POA', departure_date=dt(2021, 7, 8, 0, 0), arrive_date=dt(2021, 7, 10, 0, 0), price=100.0, carrier='GOL')
    Flight(id=UUID('50b5e710-db51-424c-b052-8383635ad39d'), origin='CWB', destination='POA', departure_date=datetime.datetime(2021, 7, 8, 0, 0), arrive_date=datetime.datetime(2021, 7, 10, 0, 0), price=100.0, carrier='GOL')
    """

    id: uuid.UUID = uuid.uuid4()
    origin: str
    destination: str
    departure_date: dt | str
    arrive_date: dt | str
    price: float
    carrier: str

    @validator('price')
    def price_must_be_positive(cls, v):
        """Validates that price is positive.
        >>> Flight(id=uuid.UUID('50b5e710-db51-424c-b052-8383635ad39d'), origin='CWB', destination='POA', departure_date=dt(2021, 7, 8, 0, 0), arrive_date=dt(2021, 7, 10, 0, 0), price=-100.0, carrier='GOL')
        Traceback (most recent call last):
          File "/usr/lib/python3.10/doctest.py", line 1350, in __run
            exec(compile(example.source, filename, "single",
          File "<doctest travel_pricing_scraper.models.Flight.price_must_be_positive[0]>", line 1, in <module>
            Flight(id=uuid.UUID('50b5e710-db51-424c-b052-8383635ad39d'), origin='CWB', destination='POA', departure_date=dt(2021, 7, 8, 0, 0), arrive_date=dt(2021, 7, 10, 0, 0), price=-100.0, carrier='GOL')
          File "pydantic/main.py", line 341, in pydantic.main.BaseModel.__init__
        pydantic.error_wrappers.ValidationError: 1 validation error for Flight
        price
          price == -100.0 must be positive (type=assertion_error)"""   # doctest: +ELLIPSIS
        assert v > 0, f'price == {v} must be positive'
        return v

    @validator('departure_date', 'arrive_date')
    def datestring_to_datetime(cls, v):
        """Converts date string to datetime.
        >>> Flight(id=uuid.UUID('50b5e710-db51-424c-b052-8383635ad39d'), origin='CWB', destination='POA', departure_date='2021-07-08', arrive_date='2021-07-10', price=100.0, carrier='GOL')
        Flight(id=UUID('...'), origin='CWB', destination='POA', departure_date=datetime.datetime(2021, 7, 8, 0, 0), arrive_date=datetime.datetime(2021, 7, 10, 0, 0), price=100.0, carrier='GOL')"""
        if isinstance(v, str):
            return dt.fromisoformat(v)
        return v
    
    @validator('price', pre=True)
    def brl_to_float(cls, v):
        """Converts BRL string to float.
        >>> Flight(id=uuid.UUID('50b5e710-db51-424c-b052-8383635ad39d'), origin='CWB', destination='POA', departure_date=dt(2021, 7, 8, 0, 0), arrive_date=dt(2021, 7, 10, 0, 0), price='100,00', carrier='GOL')
        Flight(id=UUID('...'), origin='CWB', destination='POA', departure_date=datetime.datetime(2021, 7, 8, 0, 0), arrive_date=datetime.datetime(2021, 7, 10, 0, 0), price=100.0, carrier='GOL')"""
        if isinstance(v, str):
            return float(v.replace('.', '').replace(',', '.'))
        return v



import asyncio

from ...models import Flight
from .decolar import decolar_flights


async def scrape_flights(origin, destination, flight_date):
    results = []
    tasks = []
    flights = []
    tasks.append(
        asyncio.create_task(decolar_flights(origin, destination, flight_date))
    )
    await asyncio.wait(tasks)
    results = [task.result() for task in tasks]
    for result in results:
        for flight in result:
            flights.append(flight)

    return [Flight(**flight) for flight in flights]
